! Description:
!> @file
!!   Integrate the radiative transfer equation for solar radiation
!
!> @brief
!!   Integrate the radiative transfer equation for solar radiation
!!
!! @details
!!   This is the clear-sky RTTOV solver for solar radiation. This includes
!!   the Rayleigh single-scattering parameterisation which may be used
!!   with other solvers (e.g. the DOM solar solver unless the
!!   rayleigh_multi_scatt option is set).
!!
!!   For v9 predictor coefficients, Rayleigh extinction is included in the
!!   gas optical depths. For v13 predictor coefficients, the Rayleigh
!!   extinction is computed separately within RTTOV. Rayleigh scattering is
!!   computed using the parameterisation in Bucholtz (1995). For v9 predictors,
!!   the parameterised scattering cross-section is used. For v13 predictors,
!!   the parameterised extinction coefficient is used (as in
!!   rttov_rayleigh_extinction.F90).
!!
!!   Rayleigh single scattering is computed for UV/VIS/NIR channels only.
!!
!!   The direct surface-reflected radiance term is computed here for all solar-
!!   affected channels. Note that this subroutine must be called before the
!!   rttov_integrate_thermal subroutine is called so that this term is included
!!   in the total radiances for mixed thermal+solar channels. This applies only
!!   to clear-sky simulations.
!!
!!   The overcast radiances are computed for UV/VIS/NIR channels only. Mixed
!!   thermal+solar channels are treated in rttov_integrate_thermal.
!!
!!   Finally the total radiances are computed for the UV/VIS/NIR channels only.
!!   Mixed thermal+solar channels are treated in rttov_integrate_thermal.
!!
!!
!!   Eyre J.R. 1991 A fast radiative transfer model for satellite sounding
!!   systems. ECMWF Research Dept. Tech. Memo. 176
!!
!!   Saunders R.W., M. Matricardi and P. Brunel 1999 An Improved Fast Radiative
!!   Transfer Model for Assimilation of Satellite Radiance Observations.
!!   QJRMS, 125, 1407-1425.
!!
!!   Matricardi, M. 2003 RTIASI-4, a new version of the ECMWF fast radiative
!!   transfer model for the infrared atmospheric sounding interferometer.
!!   ECMWF Research Dept. Tech. Memo. 425
!!
!!   Matricardi, M. 2005 The inclusion of aerosols and clouds in RTIASI,the
!!   ECMWF radiative transfer model for the infrared atmospheric sounding
!!   interferometer. ECMWF Research Dept. Tech. Memo. 474
!!
!!   Bucholtz, A., 1995: Rayleigh-scattering calculations for the terrestrial
!!   atmosphere. Applied Optics, 34, 15, 2765-2773.
!!
!!
!! @param[in]     opts                  RTTOV options structure
!! @param[in]     coef                  optical depth coefficients structure
!! @param[in]     chanprof              specifies channels and profiles to simulate
!! @param[in]     thermal               per-channel flag to indicate if emissive simulations are being performed
!! @param[in]     solar                 per-channel flag to indicate if solar simulations are being performed
!! @param[in]     brdf                  surface BRDFs for direct solar beam
!! @param[in]     refl_norm             surface relfectance normalisation factors
!! @param[in]     diffuse_refl          surface reflectance for downwelling radiation
!! @param[in]     refl_cloud_top_in     cloud top reflectance for simple cloud scheme, optional
!! @param[in]     solar_spectrum        TOA solar irradiance for each channel
!! @param[in]     profiles              input atmospheric profiles and surface variables
!! @param[in]     raytracing            raytracing structure
!! @param[in]     maxncolumns           largest number of cloud columns across all profiles
!! @param[in]     cloud_columns         computed cloud column data
!! @param[in]     transmission_aux      top-level auxiliary transmission structure
!! @param[in,out] auxrad_column         internal radiance structure
!! @param[in,out] rad                   primary output radiance structure
!!
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 September 2021, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2024, EUMETSAT, All Rights Reserved.
!
SUBROUTINE rttov_integrate_solar( &
              opts,              &
              coef,              &
              chanprof,          &
              thermal,           &
              solar,             &
              brdf,              &
              refl_norm,         &
              diffuse_refl,      &
              refl_cloud_top_in, &
              solar_spectrum,    &
              profiles,          &
              raytracing,        &
              maxncolumns,       &
              cloud_columns,     &
              transmission_aux,  &
              auxrad_column,     &
              rad)

  USE rttov_kinds, ONLY : jpim, jprv, jplm

  USE rttov_types, ONLY : rttov_options, rttov_coef, rttov_chanprof, &
                          rttov_profile_internal, &
                          rttov_raytracing, rttov_cloud_columns, &
                          rttov_transmission_aux, rttov_radiance_aux, &
                          rttov_radiance
!INTF_OFF
  USE rttov_const, ONLY : min_tau, pi_r, z4pi_r, &
                          deg2rad, gravity, na, rgc, mair, &
                          overcast_albedo_wvn, overcast_albedo1, overcast_albedo2, &
                          ray_ps, ray_ts, ray_scs_wlm, &
                          ray_scs_a1x, ray_scs_b1, ray_scs_c1, ray_scs_d1, &
                          ray_scs_a2x, ray_scs_b2, ray_scs_c2, ray_scs_d2

  USE yomhook, ONLY : lhook, dr_hook, jphook
!INTF_ON
  IMPLICIT NONE

  TYPE(rttov_options),          INTENT(IN)              :: opts
  TYPE(rttov_coef),             INTENT(IN)              :: coef
  TYPE(rttov_chanprof),         INTENT(IN)              :: chanprof(:)
  LOGICAL(jplm),                INTENT(IN)              :: thermal(:)
  LOGICAL(jplm),                INTENT(IN)              :: solar(:)
  REAL(jprv),                   INTENT(IN)              :: brdf(:)
  REAL(jprv),                   INTENT(IN)              :: refl_norm(:)
  REAL(jprv),                   INTENT(IN)              :: diffuse_refl(:)
  REAL(jprv),                   INTENT(IN),   OPTIONAL  :: refl_cloud_top_in(:)
  REAL(jprv),                   INTENT(IN)              :: solar_spectrum(:)
  TYPE(rttov_profile_internal), INTENT(IN)              :: profiles(:)
  TYPE(rttov_raytracing),       INTENT(IN)              :: raytracing
  INTEGER(jpim),                INTENT(IN)              :: maxncolumns
  TYPE(rttov_cloud_columns),    INTENT(IN)              :: cloud_columns
  TYPE(rttov_transmission_aux), INTENT(IN)              :: transmission_aux
  TYPE(rttov_radiance_aux),     INTENT(INOUT)           :: auxrad_column
  TYPE(rttov_radiance),         INTENT(INOUT)           :: rad
!INTF_END

  INTEGER(jpim) :: i, lev, col, lay, nchanprof, nlayers, nlevels
  LOGICAL(jplm) :: key_rad_only, do_scatt
  REAL(jprv)    :: refl, refl_norm_scat
  REAL(jprv)    :: rad_up(profiles(1)%nlayers,0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: refl_cloud_top(SIZE(chanprof))
  REAL(jphook)  :: zhook_handle
!- End of header ------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_SOLAR',0_jpim,ZHOOK_HANDLE)

! Define macros for commonly used variables
#define prof chanprof(i)%Prof
#define chan chanprof(i)%Chan

  !-------------------------------------------------------------------------------
  ! Initialise useful variables
  !-------------------------------------------------------------------------------

  nchanprof = SIZE(chanprof)
  nlayers = profiles(1)%nlayers
  nlevels = nlayers + 1

  do_scatt = opts%scatt%hydrometeors .OR. opts%scatt%aerosols
  key_rad_only = opts%pcrttov%enable_pcrttov .OR. do_scatt

  rad_up(:,:,:) = 0._jprv
  auxrad_column%rad_down_solar(:,:,:) = 0._jprv

  IF (PRESENT(refl_cloud_top_in)) THEN
    refl_cloud_top(:) = refl_cloud_top_in(:)
  ELSE
    refl_cloud_top(:) = 0._jprv
  ENDIF

  !-------------------------------------------------------------------------------
  ! Calculate clear-sky Rayleigh scattering contribution from layers
  !-------------------------------------------------------------------------------
  IF (opts%rt_all%rayleigh_single_scatt) THEN
    CALL rayleigh_single_scattering(raytracing, profiles, transmission_aux, &
                                    auxrad_column)

    ! Add solar atmospheric contributions to the clear and cloudy columns
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE
      ! Downward-scattered component: this radiation is travelling along the
      ! satellite line-of-sight. diffuse_refl must be divided by pi to give
      ! a BRDF and multiplied by cos(sat_zen_angle)
      refl_norm_scat = COS(profiles(prof)%zenangle * deg2rad) * pi_r

      DO col = 0, cloud_columns%ncolumn(prof)
        auxrad_column%rad_column(col,i) = &
            rad_up(nlayers,col,i) + &
            auxrad_column%rad_down_solar(nlayers,col,i) * &
            diffuse_refl(i) * refl_norm_scat * &
            transmission_aux%solar_path1%tau_level(nlevels,col,i)**2_jpim
      ENDDO
    ENDDO
  ENDIF

  !-------------------------------------------------------------------------------
  ! Solar surface contribution
  !-------------------------------------------------------------------------------
  ! Clear-sky simulations, all solar-affected channels

  IF (.NOT. do_scatt) &
    CALL solar_surface_contribution(transmission_aux, brdf, auxrad_column)

  !-------------------------------------------------------------------------------
  ! Calculate overcast radiances
  !-------------------------------------------------------------------------------
  ! Thermal channels (including solar-affected ones) are handled by rttov_integrate_thermal

  IF (.NOT. key_rad_only) THEN
    ! rad%overcast includes solar contribution ONLY for pure-solar channels because assumed
    !   cloud emissivity is 1.0 (i.e. no reflection) for all thermal channels
    col = 0
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE
      ! Very crude model: assumes clouds are Lambertian reflectors with fixed albedo
      ! Use input cloud top BRDF if user has supplied it, otherwise use default BRDF
      IF (refl_cloud_top(i) > 0) THEN
        refl = refl_cloud_top(i)
      ELSE
        IF (coef%ff_cwn(chan) > overcast_albedo_wvn) THEN
          refl = overcast_albedo1 * pi_r
        ELSE
          refl = overcast_albedo2 * pi_r
        ENDIF
      ENDIF
      DO lay = 1, nlayers
        lev = lay + 1
        ! Overcast radiances at given cloud top
        rad%overcast(lay,i) = solar_spectrum(i) * refl / raytracing%pathsun(lay,prof) * &
                              transmission_aux%solar_path2%tau_level(lev,col,i) + &
                              rad_up(lay,col,i) + &
                              auxrad_column%rad_down_solar(lay,col,i) * &
                              refl / raytracing%pathsat(lay,prof) * &
                              transmission_aux%solar_path1%tau_level(lev,col,i) ** 2_jpim
      ENDDO
    ENDDO
  ENDIF

  !-------------------------------------------------------------------------------
  ! Calculate total radiance
  !-------------------------------------------------------------------------------
  ! Thermal channels (including solar-affected ones) are handled by rttov_integrate_thermal

  WHERE (solar(1:nchanprof) .AND. .NOT. thermal(1:nchanprof)) &
    rad%clear(1:nchanprof) = auxrad_column%rad_column(0,1:nchanprof)

  IF (do_scatt) THEN
    !---------------------------------------------------
    ! Calculate Rayleigh single scattering radiance contributions for scattering radiances
    !---------------------------------------------------
    ! Additional radiance contributions are calculated in other solvers
    IF (opts%rt_all%rayleigh_single_scatt) THEN
      DO i = 1, nchanprof
        IF (thermal(i) .OR. .NOT. solar(i)) CYCLE

        DO col = 1, cloud_columns%ncolumn(prof)
          rad%total(i) = rad%total(i) + &
                         auxrad_column%rad_column(col,i) * &
                         (cloud_columns%xcol(col+1,prof) - cloud_columns%xcol(col,prof))
        ENDDO

        rad%total(i) = rad%total(i) + &
                       auxrad_column%rad_column(0,i) * cloud_columns%xcolclr(prof)
      ENDDO
    ENDIF
  ELSE
    !---------------------------------------------------
    ! Calculate total radiance (clear case/simple cloud)
    !---------------------------------------------------
    ! Interpolate to given cloud-top pressures
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE

      ! Overcast radiances are for nlayers with opaque cloud at lower bounding level
      lay = profiles(prof)%ctp_layer
      rad%cloudy(i) = rad%overcast(lay-1,i) + profiles(prof)%ctp_pfraction * &
                      (rad%overcast(lay,i) - rad%overcast(lay-1,i))  

      rad%total(i) = rad%clear(i) + profiles(prof)%cfraction * &
                                    (rad%cloudy(i) - rad%clear(i))
    ENDDO
  ENDIF

  IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_SOLAR',1_jpim,ZHOOK_HANDLE)

CONTAINS

  SUBROUTINE rayleigh_single_scattering(raytracing, profiles, transmission_aux, &
                                        auxrad_column)

    TYPE(rttov_raytracing),       INTENT(IN)    :: raytracing
    TYPE(rttov_profile_internal), INTENT(IN)    :: profiles(:)
    TYPE(rttov_transmission_aux), INTENT(IN)    :: transmission_aux
    TYPE(rttov_radiance_aux),     INTENT(INOUT) :: auxrad_column

    INTEGER(jpim) :: i, col, lay, lev
    REAL(jprv) :: wlm, ss_param
    REAL(jprv) :: cosscata_term1, cosscata_term2, cosscata
    REAL(jprv) :: gamma, ray_phase, solar_src, solar_src_updn
    REAL(jprv) :: rayrad_up(0:nlayers,0:maxncolumns), rayrad_dn(0:nlayers,0:maxncolumns)

    ! The solar geometry is approximated using the sun-surface path values,
    ! and similarly for the sun-level-satellite transmittances.

    ! Rayleigh scattering only included for channels less than rayleigh_max_wavelength option.

    DO i = 1, nchanprof
      wlm = 10000._jprv / coef%ff_cwn(chan)    ! Wavelength in microns
      IF (.NOT. solar(i) .OR. wlm > opts%rt_all%rayleigh_max_wavelength) CYCLE

      ! Calculate layer-independent scattering parameter
      IF (coef%fmv_model_ver <= 9) THEN
        IF (wlm < ray_scs_wlm) THEN
          ss_param = ray_scs_a1x * wlm ** (ray_scs_b1 + ray_scs_c1 * wlm + ray_scs_d1/wlm)
        ELSE
          ss_param = ray_scs_a2x * wlm ** (ray_scs_b2 + ray_scs_c2 * wlm + ray_scs_d2/wlm)
        ENDIF
        ss_param = ss_param * 0.01_jprv ** 2_jpim * 1.E3_jprv * 100._jprv * na * z4pi_r / (gravity * mair)
      ELSE
        ss_param = coef%ss_rayleigh_ext(chan) * (ray_ts / ray_ps) * rgc * z4pi_r / (gravity * mair)
      ENDIF
      gamma = coef%rayleigh_depol_gamma(chan)

      rayrad_up(0,:) = 0._jprv
      rayrad_dn(0,:) = 0._jprv

      ! Sum contributions from atmospheric layers
      DO lev = 2, nlevels
        lay = lev - 1

        ! Skip calculation for whole layers above min pressure
        IF (profiles(prof)%p_half(lev) <= opts%rt_all%rayleigh_min_pressure) THEN
          rayrad_up(lay,:) = 0._jprv
          rayrad_dn(lay,:) = 0._jprv
          CYCLE
        ENDIF

        solar_src = solar_spectrum(i) * & ! mW m^-2 (cm^-1)^-1
              (profiles(prof)%p_half(lev) - profiles(prof)%p_half(lev-1)) * &
              raytracing%pathsat(lay, prof) * ss_param

        ! cosine of scattering angle - raytracing%zasat/zasun contain the sine of the angles
        cosscata_term1 = SQRT((1._jprv - raytracing%zasat(lay, prof) * raytracing%zasat(lay, prof)) * &
                         (1._jprv - raytracing%zasun(lay, prof) * raytracing%zasun(lay, prof)))
        cosscata_term2 = raytracing%zasat(lay, prof) * raytracing%zasun(lay, prof) * &
                         COS((profiles(prof)%azangle - profiles(prof)%sunazangle)*deg2rad)

        cosscata = - cosscata_term1 - cosscata_term2
        ray_phase = (1._jprv + 3._jprv * gamma + (1._jprv - gamma) * cosscata**2) * &
                    0.75_jprv / (1._jprv + 2._jprv * gamma)
        solar_src_updn = solar_src * ray_phase

        ! Phase function symmetry means upwelling and downwelling radiances are the same
        DO col = 0, cloud_columns%ncolumn(prof)
          rayrad_up(lay,col) = rayrad_up(lay-1,col) + solar_src_updn * &
                               transmission_aux%solar_path2%tau_level(lev-1,col,i)
        ENDDO

        DO col = 0, cloud_columns%ncolumn(prof)
          IF (transmission_aux%solar_path1%tau_level(lev-1,col,i) > min_tau) THEN
            rayrad_dn(lay,col) = rayrad_dn(lay-1,col) + solar_src_updn * &
                                 transmission_aux%solar_path2%tau_level(lev-1,col,i) / &
                                 transmission_aux%solar_path1%tau_level(lev-1,col,i) ** 3_jpim
          ELSE
            rayrad_dn(lay,col) = rayrad_dn(lay-1,col)
          ENDIF
        ENDDO
      ENDDO

      ! Add Rayleigh contributions to radiance totals
      DO col = 0, cloud_columns%ncolumn(prof)
        rad_up(:,col,i)                       = rayrad_up(1:nlayers,col)
        auxrad_column%rad_down_solar(:,col,i) = rayrad_dn(1:nlayers,col)
      ENDDO
    ENDDO
  END SUBROUTINE rayleigh_single_scattering

  SUBROUTINE solar_surface_contribution(transmission_aux, brdf, auxrad_column)

    TYPE(rttov_transmission_aux), INTENT(IN)    :: transmission_aux
    REAL(jprv),                   INTENT(IN)    :: brdf(:)
    TYPE(rttov_radiance_aux),     INTENT(INOUT) :: auxrad_column

    INTEGER(jpim) :: i

    col = 0
    DO i = 1, nchanprof
      IF (.NOT. solar(i)) CYCLE
      auxrad_column%rad_column(col,i) = auxrad_column%rad_column(col,i) + &
              solar_spectrum(i) * brdf(i) * refl_norm(i) * &
              transmission_aux%solar_path2%tau_level(nlevels,col,i)
    ENDDO
  END SUBROUTINE solar_surface_contribution

END SUBROUTINE rttov_integrate_solar
