! Description:
!> @file
!!   K of integration of the radiative transfer equation for solar radiation
!
!> @brief
!!   K of integration of the radiative transfer equation for solar radiation
!!
!!
!! @param[in]     opts                        RTTOV options structure
!! @param[in]     coef                        optical depth coefficients structure
!! @param[in]     chanprof                    specifies channels and profiles to simulate
!! @param[in]     thermal                     per-channel flag to indicate if emissive simulations are being performed
!! @param[in]     solar                       per-channel flag to indicate if solar simulations are being performed
!! @param[in]     brdf                        surface BRDFs for direct solar beam
!! @param[in,out] brdf_k                      surface BRDF increments for direct solar beam
!! @param[in]     refl_norm                   surface relfectance normalisation factors
!! @param[in]     diffuse_refl                surface reflectance for downwelling radiation
!! @param[in,out] diffuse_refl_k              surface reflectance increments for downwelling radiation
!! @param[in]     refl_cloud_top_in           cloud top reflectance for simple cloud scheme, optional
!! @param[in]     solar_spectrum              TOA solar irradiance for each channel
!! @param[in]     profiles                    input atmospheric profiles and surface variables
!! @param[in,out] profiles_k                  input atmospheric profile increments
!! @param[in]     raytracing                  raytracing structure
!! @param[in,out] raytracing_k                raytracing increments
!! @param[in]     maxncolumns                 largest number of cloud columns across all profiles
!! @param[in]     cloud_columns               computed cloud column data
!! @param[in,out] cloud_columns_k             cloud column increments
!! @param[in]     transmission_aux            top-level auxiliary transmission structure
!! @param[in,out] transmission_aux_k          auxiliary transmission increments
!! @param[in]     auxrad_column               internal radiance structure
!! @param[in,out] auxrad_column_k             internal radiance increments
!! @param[in]     rad                         primary output radiance structure
!! @param[in,out] rad_k                       output radiance increments
!!
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 September 2021, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2024, EUMETSAT, All Rights Reserved.
!
SUBROUTINE rttov_integrate_solar_k( &
              opts,                &
              coef,                &
              chanprof,            &
              thermal,             &
              solar,               &
              brdf,                &
              brdf_k,              &
              refl_norm,           &
              diffuse_refl,        &
              diffuse_refl_k,      &
              refl_cloud_top_in,   &
              solar_spectrum,      &
              profiles,            &
              profiles_k,          &
              raytracing,          &
              raytracing_k,        &
              maxncolumns,         &
              cloud_columns,       &
              cloud_columns_k,     &
              transmission_aux,    &
              transmission_aux_k,  &
              auxrad_column,       &
              auxrad_column_k,     &
              rad,                 &
              rad_k)

  USE rttov_kinds, ONLY : jpim, jprv, jplm

  USE rttov_types, ONLY : rttov_options, rttov_coef, rttov_chanprof, &
                          rttov_profile_internal, &
                          rttov_raytracing, rttov_cloud_columns, &
                          rttov_transmission_aux, rttov_radiance_aux, &
                          rttov_radiance
!INTF_OFF
  USE rttov_const, ONLY : min_tau, pi_r, z4pi_r, &
                          deg2rad, gravity, na, rgc, mair, &
                          overcast_albedo_wvn, overcast_albedo1, overcast_albedo2, &
                          ray_ps, ray_ts, ray_scs_wlm, &
                          ray_scs_a1x, ray_scs_b1, ray_scs_c1, ray_scs_d1, &
                          ray_scs_a2x, ray_scs_b2, ray_scs_c2, ray_scs_d2

  USE yomhook, ONLY : lhook, dr_hook, jphook
!INTF_ON
  IMPLICIT NONE

  TYPE(rttov_options),                INTENT(IN)              :: opts
  TYPE(rttov_coef),                   INTENT(IN)              :: coef
  TYPE(rttov_chanprof),               INTENT(IN)              :: chanprof(:)
  LOGICAL(jplm),                      INTENT(IN)              :: thermal(:)
  LOGICAL(jplm),                      INTENT(IN)              :: solar(:)
  REAL(jprv),                         INTENT(IN)              :: brdf(:)
  REAL(jprv),                         INTENT(INOUT)           :: brdf_k(:)
  REAL(jprv),                         INTENT(IN)              :: refl_norm(:)
  REAL(jprv),                         INTENT(IN)              :: diffuse_refl(:)
  REAL(jprv),                         INTENT(INOUT)           :: diffuse_refl_k(:)
  REAL(jprv),                         INTENT(IN),   OPTIONAL  :: refl_cloud_top_in(:)
  REAL(jprv),                         INTENT(IN)              :: solar_spectrum(:)
  TYPE(rttov_profile_internal),       INTENT(IN)              :: profiles(:)
  TYPE(rttov_profile_internal),       INTENT(INOUT)           :: profiles_k(:)
  TYPE(rttov_raytracing),             INTENT(IN)              :: raytracing
  TYPE(rttov_raytracing),             INTENT(INOUT)           :: raytracing_k
  INTEGER(jpim),                      INTENT(IN)              :: maxncolumns
  TYPE(rttov_cloud_columns),          INTENT(IN)              :: cloud_columns
  TYPE(rttov_cloud_columns),          INTENT(INOUT)           :: cloud_columns_k
  TYPE(rttov_transmission_aux),       INTENT(IN)              :: transmission_aux
  TYPE(rttov_transmission_aux),       INTENT(INOUT)           :: transmission_aux_k
  TYPE(rttov_radiance_aux),           INTENT(IN)              :: auxrad_column
  TYPE(rttov_radiance_aux),           INTENT(INOUT)           :: auxrad_column_k
  TYPE(rttov_radiance),               INTENT(IN)              :: rad
  TYPE(rttov_radiance),               INTENT(INOUT)           :: rad_k
!INTF_END

  INTEGER(jpim) :: i, lev, col, lay, nchanprof, nlayers, nlevels
  LOGICAL(jplm) :: key_rad_only, do_scatt
  REAL(jprv)    :: refl, refl_norm_scat
  REAL(jprv)    :: rad_up_k(profiles(1)%nlayers,0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: rad_down_k(profiles(1)%nlayers,0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: rad_up_total_k(0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: rad_down_total_k(0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: refl_cloud_top(SIZE(chanprof))
  REAL(jphook)  :: zhook_handle
!- End of header ------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_SOLAR_K',0_jpim,ZHOOK_HANDLE)

#define prof chanprof(i)%Prof
#define chan chanprof(i)%Chan

  !-------------------------------------------------------------------------------
  ! Initialise useful variables
  !-------------------------------------------------------------------------------

  nchanprof = SIZE(chanprof)
  nlayers = profiles(1)%nlayers
  nlevels = nlayers + 1

  do_scatt = opts%scatt%hydrometeors .OR. opts%scatt%aerosols
  key_rad_only = opts%pcrttov%enable_pcrttov .OR. do_scatt

  rad_up_k(:,:,:)       = 0._jprv
  rad_down_k(:,:,:)     = 0._jprv
  ! rad_up_total_k(:,:)   = 0._jprv
  ! rad_down_total_k(:,:) = 0._jprv

  IF (PRESENT(refl_cloud_top_in)) THEN
    refl_cloud_top(:) = refl_cloud_top_in(:)
  ELSE
    refl_cloud_top(:) = 0._jprv
  ENDIF

  !-------------------------------------------------------------------------------
  ! Calculate total radiance
  !-------------------------------------------------------------------------------

  IF (do_scatt) THEN
    !---------------------------------------------------
    ! Calculate Rayleigh single scattering radiance contributions for scattering radiances
    !---------------------------------------------------
    IF (opts%rt_all%rayleigh_single_scatt) THEN
      DO i = 1, nchanprof
        IF (thermal(i) .OR. .NOT. solar(i)) CYCLE

        auxrad_column_k%rad_column(0,i) = auxrad_column_k%rad_column(0,i) + &
                                          rad_k%total(i) * cloud_columns%xcolclr(prof)
        cloud_columns_k%xcolclr(i) = cloud_columns_k%xcolclr(i) + &
                                    rad_k%total(i) * auxrad_column%rad_column(0,i)

        DO col = cloud_columns%ncolumn(prof), 1, -1 !reverse loop nec. due to data dependence
          auxrad_column_k%rad_column(col,i) = auxrad_column_k%rad_column(col,i) + &
            rad_k%total(i) * (cloud_columns%xcol(col+1,prof) - cloud_columns%xcol(col,prof))

          cloud_columns_k%xcol(col+1,i) = cloud_columns_k%xcol(col+1,i) + &
            rad_k%total(i) * auxrad_column%rad_column(col,i)

          cloud_columns_k%xcol(col,i) = cloud_columns_k%xcol(col,i) - &
            rad_k%total(i) * auxrad_column%rad_column(col,i)
        ENDDO
      ENDDO
    ENDIF
  ELSE
    !---------------------------------------------------
    ! Calculate total radiance (clear case/simple cloud)
    !---------------------------------------------------
    ! Interpolate to given cloud-top pressures
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE

      rad_k%clear(i)  = rad_k%clear(i) + &
                          (1._jprv - profiles(prof)%cfraction) * rad_k%total(i)
      rad_k%cloudy(i) = profiles(prof)%cfraction * rad_k%total(i)
      profiles_k(i)%cfraction = profiles_k(i)%cfraction + &
                                (rad%cloudy(i) - rad%clear(i)) * rad_k%total(i)

      lay = profiles(prof)%ctp_layer

      rad_k%overcast(lay-1,i) = & !rad_k%overcast(lay-1,i) + & !first use of rad_k%overcast
            (1._jprv - profiles(prof)%ctp_pfraction) * rad_k%cloudy(i)

      rad_k%overcast(lay,i) = & !rad_k%overcast(lay,i) + & !first use of rad_k%overcast
            profiles(prof)%ctp_pfraction * rad_k%cloudy(i)

      profiles_k(i)%ctp_pfraction = profiles_k(i)%ctp_pfraction + &
            (rad%overcast(lay,i) - rad%overcast(lay-1,i)) * rad_k%cloudy(i)
    ENDDO
  ENDIF

  WHERE (solar(1:nchanprof) .AND. .NOT. thermal(1:nchanprof)) &
    auxrad_column_k%rad_column(0,1:nchanprof) = &
      auxrad_column_k%rad_column(0,1:nchanprof) + rad_k%clear(1:nchanprof)

  !-------------------------------------------------------------------------------
  ! Calculate overcast radiances
  !-------------------------------------------------------------------------------
  IF (.NOT. key_rad_only) THEN
    col = 0_jpim
    !cdir nodep
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE
      IF (refl_cloud_top(i) > 0) THEN
        refl = refl_cloud_top(i)
      ELSE
        IF (coef%ff_cwn(chan) > overcast_albedo_wvn) THEN
          refl = overcast_albedo1 * pi_r
        ELSE
          refl = overcast_albedo2 * pi_r
        ENDIF
      ENDIF
      DO lay = 1, nlayers
        lev = lay + 1
        transmission_aux_k%solar_path2%tau_level(lev,col,i) = &
          transmission_aux_k%solar_path2%tau_level(lev,col,i) + &
          solar_spectrum(i) * refl / raytracing%pathsun(lay,prof) * &
          rad_k%overcast(lay,i)
        raytracing_k%pathsun(lay,i) = raytracing_k%pathsun(lay,i) - &
          solar_spectrum(i) * refl / raytracing%pathsun(lay,prof)**2_jpim * &
          transmission_aux%solar_path2%tau_level(lev,col,i) * rad_k%overcast(lay,i)

        transmission_aux_k%solar_path1%tau_level(lev,col,i) = &
          transmission_aux_k%solar_path1%tau_level(lev,col,i) + &
          refl / raytracing%pathsat(lay,prof) * &
          auxrad_column%rad_down_solar(lay,col,i) * &
          transmission_aux%solar_path1%tau_level(lev,col,i) * &
          2_jpim * rad_k%overcast(lay,i)
        raytracing_k%pathsat(lay,i) = &
          raytracing_k%pathsat(lay,i) - &
          refl / raytracing%pathsat(lay,prof) * &
          transmission_aux%solar_path1%tau_level(lev,col,i)**2_jpim * &
          auxrad_column%rad_down_solar(lay,col,i) / raytracing%pathsat(lay,prof) * &
          rad_k%overcast(lay,i)
        rad_down_k(lay,col,i) = & ! rad_down_k(lay,col,i) + &
          refl / raytracing%pathsat(lay,prof) * &
          transmission_aux%solar_path1%tau_level(lev,col,i)**2_jpim * &
          rad_k%overcast(lay,i)

        rad_up_k(lay,col,i) = & ! rad_up_k(lay,col,i) + &
          rad_k%overcast(lay,i)
      ENDDO
    ENDDO
    ! rad_k%overcast done
    ! rad_k%clear done
  ENDIF

  rad_k%clear(:) = 0._jprv

  !-------------------------------------------------------------------------------
  ! Solar surface contribution
  !-------------------------------------------------------------------------------
  IF (.NOT. do_scatt) &
    CALL solar_surface_contribution_k(transmission_aux, transmission_aux_k, &
                                      brdf, brdf_k, auxrad_column_k)

  !-------------------------------------------------------------------------------
  ! Calculate clear-sky Rayleigh scattering contribution from layers
  !-------------------------------------------------------------------------------
  IF (opts%rt_all%rayleigh_single_scatt) THEN

    ! Add solar atmospheric contributions to the clear and cloudy columns
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE
      refl_norm_scat = COS(profiles(prof)%zenangle * deg2rad) * pi_r

      DO col = 0, cloud_columns%ncolumn(prof)
        rad_up_total_k(col,i) = & ! rad_up_total_k(col,i) + &
          auxrad_column_k%rad_column(col,i)

        diffuse_refl_k(i) = diffuse_refl_k(i) + &
          transmission_aux%solar_path1%tau_level(nlevels,col,i)**2_jpim * refl_norm_scat * &
          auxrad_column%rad_down_solar(nlayers,col,i) * auxrad_column_k%rad_column(col,i)

        rad_down_total_k(col,i) = & ! rad_down_total_k(col,i) + &
          transmission_aux%solar_path1%tau_level(nlevels,col,i)**2_jpim * &
          diffuse_refl(i) * refl_norm_scat * auxrad_column_k%rad_column(col,i)

        transmission_aux_k%solar_path1%tau_level(nlevels,col,i) = &
          transmission_aux_k%solar_path1%tau_level(nlevels,col,i) + &
          transmission_aux%solar_path1%tau_level(nlevels,col,i) * &
          diffuse_refl(i) * refl_norm_scat * 2._jprv * auxrad_column_k%rad_column(col,i) * &
          auxrad_column%rad_down_solar(nlayers,col,i)
      ENDDO
    ENDDO

    CALL rayleigh_single_scattering_k(raytracing, raytracing_k, &
                                      profiles, profiles_k, transmission_aux, &
                                      transmission_aux_k)
  ENDIF

  IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_SOLAR_K',1_jpim,ZHOOK_HANDLE)

CONTAINS

  SUBROUTINE rayleigh_single_scattering_k(raytracing, raytracing_k, &
                                          profiles, profiles_k, transmission_aux, &
                                          transmission_aux_k)

    TYPE(rttov_raytracing),       INTENT(IN)    :: raytracing
    TYPE(rttov_raytracing),       INTENT(INOUT) :: raytracing_k
    TYPE(rttov_profile_internal), INTENT(IN)    :: profiles(:)
    TYPE(rttov_profile_internal), INTENT(INOUT) :: profiles_k(:)
    TYPE(rttov_transmission_aux), INTENT(IN)    :: transmission_aux
    TYPE(rttov_transmission_aux), INTENT(INOUT) :: transmission_aux_k

    INTEGER(jpim) :: i, col, lay, lev
    REAL(jprv) :: wlm, ss_param
    REAL(jprv) :: cossat, cossol, cosazi, cosscata_term1, cosscata_term2, cosscata
    REAL(jprv) :: gamma, ray_phase, solar_src, solar_src_updn
    REAL(jprv) :: cosscata_term1_k, cosscata_term2_k, cosscata_k
    REAL(jprv) :: ray_phase_k, solar_src_k, solar_src_updn_k
    REAL(jprv) :: rayrad_up_k(0:nlayers,0:maxncolumns), rayrad_dn_k(0:nlayers,0:maxncolumns)

    DO i = 1, nchanprof
      wlm = 10000._jprv / coef%ff_cwn(chan)    ! Wavelength in microns
      IF (.NOT. solar(i) .OR. wlm > opts%rt_all%rayleigh_max_wavelength) CYCLE

      ! Calculate layer-independent scattering parameter

      IF (coef%fmv_model_ver <= 9) THEN
        IF (wlm < ray_scs_wlm) THEN
          ss_param = ray_scs_a1x * wlm ** (ray_scs_b1 + ray_scs_c1 * wlm + ray_scs_d1/wlm)
        ELSE
          ss_param = ray_scs_a2x * wlm ** (ray_scs_b2 + ray_scs_c2 * wlm + ray_scs_d2/wlm)
        ENDIF
        ss_param = ss_param * 0.01_jprv ** 2_jpim * 1.E3_jprv * 100._jprv * na * z4pi_r / (gravity * mair)
      ELSE
        ss_param = coef%ss_rayleigh_ext(chan) * (ray_ts / ray_ps) * rgc * z4pi_r / (gravity * mair)
      ENDIF
      gamma = coef%rayleigh_depol_gamma(chan)

      rayrad_up_k(:,:) = 0._jprv
      rayrad_dn_k(:,:) = 0._jprv

      ! Atmospheric contribution
      DO col = 0, cloud_columns%ncolumn(prof)
        rayrad_dn_k(nlayers,col)   = rayrad_dn_k(nlayers,col) + rad_down_total_k(col,i)
        rayrad_dn_k(1:nlayers,col) = rayrad_dn_k(1:nlayers,col) + rad_down_k(:,col,i)
        rayrad_up_k(nlayers,col)   = rayrad_up_k(nlayers,col) + rad_up_total_k(col,i)
        rayrad_up_k(1:nlayers,col) = rayrad_up_k(1:nlayers,col) + rad_up_k(:,col,i)
      ENDDO

      DO lev = nlevels, 2, -1
        ! Skip calculation for whole layers above min pressure
        IF (profiles(prof)%p_half(lev) <= opts%rt_all%rayleigh_min_pressure) CYCLE

        lay = lev - 1

        solar_src = solar_spectrum(i) * &
              (profiles(prof)%p_half(lev) - profiles(prof)%p_half(lev-1)) * &
              raytracing%pathsat(lay, prof) * ss_param

        cossat = 1._jprv - raytracing%zasat(lay, prof) * raytracing%zasat(lay, prof)
        cossol = 1._jprv - raytracing%zasun(lay, prof) * raytracing%zasun(lay, prof)
        cosscata_term1 = SQRT(cossat * cossol)
        cosazi = COS((profiles(prof)%azangle - profiles(prof)%sunazangle)*deg2rad)
        cosscata_term2 = raytracing%zasat(lay, prof) * raytracing%zasun(lay, prof) * cosazi

        cosscata = - cosscata_term1 - cosscata_term2
        ray_phase = (1._jprv + 3._jprv * gamma + (1._jprv - gamma) * cosscata**2) * &
                    0.75_jprv / (1._jprv + 2._jprv * gamma)
        solar_src_updn = solar_src * ray_phase

        cosscata_term1_k = 0._jprv
        cosscata_term2_k = 0._jprv
        solar_src_updn_k = 0._jprv

        DO col = 0, cloud_columns%ncolumn(prof)
          IF (transmission_aux%solar_path1%tau_level(lev-1,col,i) > min_tau) THEN
            solar_src_updn_k = solar_src_updn_k + rayrad_dn_k(lay,col) * &
              transmission_aux%solar_path2%tau_level(lev-1,col,i) / &
              transmission_aux%solar_path1%tau_level(lev-1,col,i) ** 3_jpim

            transmission_aux_k%solar_path2%tau_level(lev-1,col,i) = &
              transmission_aux_k%solar_path2%tau_level(lev-1,col,i) + &
              rayrad_dn_k(lay,col) * solar_src_updn / &
              transmission_aux%solar_path1%tau_level(lev-1,col,i) ** 3_jpim

            transmission_aux_k%solar_path1%tau_level(lev-1,col,i) = &
              transmission_aux_k%solar_path1%tau_level(lev-1,col,i) - &
              rayrad_dn_k(lay,col) * solar_src_updn * &
              3_jpim * transmission_aux%solar_path2%tau_level(lev-1,col,i) / &
              transmission_aux%solar_path1%tau_level(lev-1,col,i) ** 4_jpim
          ENDIF

          rayrad_dn_k(lay-1,col) = rayrad_dn_k(lay-1,col) + rayrad_dn_k(lay,col)

          solar_src_updn_k = solar_src_updn_k + rayrad_up_k(lay,col) * &
            transmission_aux%solar_path2%tau_level(lev-1,col,i)

          transmission_aux_k%solar_path2%tau_level(lev-1,col,i) = &
            transmission_aux_k%solar_path2%tau_level(lev-1,col,i) + &
            solar_src_updn * rayrad_up_k(lay,col)

          rayrad_up_k(lay-1,col) = rayrad_up_k(lay-1,col) + rayrad_up_k(lay,col)
        ENDDO

        ray_phase_k = solar_src * solar_src_updn_k  ! First use of ray_phase_k, no accumulation
        solar_src_k = solar_src_updn_k * ray_phase  ! First use of solar_src_k
        cosscata_k = 2._jprv * (1._jprv - gamma) * ray_phase_k * cosscata * & ! First use of cossacata_k
                     0.75_jprv / (1._jprv + 2._jprv * gamma)
        cosscata_term1_k = cosscata_term1_k - cosscata_k
        cosscata_term2_k = cosscata_term2_k - cosscata_k

        raytracing_k%zasat(lay, i) = raytracing_k%zasat(lay, i) + &
          cosscata_term2_k * raytracing%zasun(lay, prof) * cosazi
        raytracing_k%zasun(lay, i) = raytracing_k%zasun(lay, i) + &
          cosscata_term2_k * raytracing%zasat(lay, prof) * cosazi

        raytracing_k%zasat(lay, i) = raytracing_k%zasat(lay, i) - &
          cosscata_term1_k * raytracing%zasat(lay, prof) * cossol / cosscata_term1
        raytracing_k%zasun(lay, i) = raytracing_k%zasun(lay, i) - &
          cosscata_term1_k * raytracing%zasun(lay, prof) * cossat / cosscata_term1

        IF (opts%interpolation%pressure_gradients) THEN
          profiles_k(i)%p_half(lev) = profiles_k(i)%p_half(lev) + &
              solar_src_k * solar_spectrum(i) * ss_param * &
              raytracing%pathsat(lay, prof)
          profiles_k(i)%p_half(lev-1) = profiles_k(i)%p_half(lev-1) - &
              solar_src_k * solar_spectrum(i) * ss_param * &
              raytracing%pathsat(lay, prof)
        ENDIF

        raytracing_k%pathsat(lay, i) = raytracing_k%pathsat(lay, i) + &
            solar_src_k * solar_spectrum(i) * ss_param * &
            (profiles(prof)%p_half(lev) - profiles(prof)%p_half(lev-1))
      ENDDO
    ENDDO
  END SUBROUTINE rayleigh_single_scattering_k

  SUBROUTINE solar_surface_contribution_k(transmission_aux, transmission_aux_k, &
                                          brdf, brdf_k, auxrad_column_k)

    TYPE(rttov_transmission_aux), INTENT(IN)    :: transmission_aux
    TYPE(rttov_transmission_aux), INTENT(INOUT) :: transmission_aux_k
    REAL(jprv),                   INTENT(IN)    :: brdf(:)
    REAL(jprv),                   INTENT(INOUT) :: brdf_k(:)
    TYPE(rttov_radiance_aux),     INTENT(IN)    :: auxrad_column_k

    INTEGER(jpim) :: i, col

    col = 0
    DO i = 1, nchanprof
      IF (.NOT. solar(i)) CYCLE
      brdf_k(i) = brdf_k(i) + solar_spectrum(i) * refl_norm(i) * &
        auxrad_column_k%rad_column(col,i) * transmission_aux%solar_path2%tau_level(nlevels,col,i)

      transmission_aux_k%solar_path2%tau_level(nlevels,col,i) = &
        transmission_aux_k%solar_path2%tau_level(nlevels,col,i) + &
        solar_spectrum(i) * refl_norm(i) * auxrad_column_k%rad_column(col,i) * brdf(i)
    ENDDO
  END SUBROUTINE solar_surface_contribution_k

END SUBROUTINE rttov_integrate_solar_k
