! Description:
!> @file
!!   TL of integration of the radiative transfer equation for solar radiation
!
!> @brief
!!   TL of integration of the radiative transfer equation for solar radiation
!!
!!
!! @param[in]     opts                        RTTOV options structure
!! @param[in]     coef                        optical depth coefficients structure
!! @param[in]     chanprof                    specifies channels and profiles to simulate
!! @param[in]     thermal                     per-channel flag to indicate if emissive simulations are being performed
!! @param[in]     solar                       per-channel flag to indicate if solar simulations are being performed
!! @param[in]     brdf                        surface BRDFs for direct solar beam
!! @param[in]     brdf_tl                     surface BRDF perturbations for direct solar beam
!! @param[in]     refl_norm                   surface relfectance normalisation factors
!! @param[in]     diffuse_refl                surface reflectance for downwelling radiation
!! @param[in]     diffuse_refl_tl             surface reflectance perturbations for downwelling radiation
!! @param[in]     refl_cloud_top_in           cloud top reflectance for simple cloud scheme, optional
!! @param[in]     solar_spectrum              TOA solar irradiance for each channel
!! @param[in]     profiles                    input atmospheric profiles and surface variables
!! @param[in]     profiles_tl                 input atmospheric profile perturbations
!! @param[in]     raytracing                  raytracing structure
!! @param[in]     raytracing_tl               raytracing perturbations
!! @param[in]     maxncolumns                 largest number of cloud columns across all profiles
!! @param[in]     cloud_columns               computed cloud column data
!! @param[in]     cloud_columns_tl            cloud column perturbations
!! @param[in]     transmission_aux            top-level auxiliary transmission structure
!! @param[in]     transmission_aux_tl         auxiliary transmission perturbations
!! @param[in]     auxrad_column               internal radiance structure
!! @param[in,out] auxrad_column_tl            internal radiance perturbations
!! @param[in]     rad                         primary output radiance structure
!! @param[in,out] rad_tl                      output radiance perturbations
!!
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 7 September 2021, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, DWD and MeteoFrance.
!
!    Copyright 2024, EUMETSAT, All Rights Reserved.
!
SUBROUTINE rttov_integrate_solar_tl( &
              opts,                &
              coef,                &
              chanprof,            &
              thermal,             &
              solar,               &
              brdf,                &
              brdf_tl,             &
              refl_norm,           &
              diffuse_refl,        &
              diffuse_refl_tl,     &
              refl_cloud_top_in,   &
              solar_spectrum,      &
              profiles,            &
              profiles_tl,         &
              raytracing,          &
              raytracing_tl,       &
              maxncolumns,         &
              cloud_columns,       &
              cloud_columns_tl,    &
              transmission_aux,    &
              transmission_aux_tl, &
              auxrad_column,       &
              auxrad_column_tl,    &
              rad,                 &
              rad_tl)

  USE rttov_kinds, ONLY : jpim, jprv, jplm

  USE rttov_types, ONLY : rttov_options, rttov_coef, rttov_chanprof, &
                          rttov_profile_internal, &
                          rttov_raytracing, rttov_cloud_columns, &
                          rttov_transmission_aux, rttov_radiance_aux, &
                          rttov_radiance
!INTF_OFF
  USE rttov_const, ONLY : min_tau, pi_r, z4pi_r, &
                          deg2rad, gravity, na, rgc, mair, &
                          overcast_albedo_wvn, overcast_albedo1, overcast_albedo2, &
                          ray_ps, ray_ts, ray_scs_wlm, &
                          ray_scs_a1x, ray_scs_b1, ray_scs_c1, ray_scs_d1, &
                          ray_scs_a2x, ray_scs_b2, ray_scs_c2, ray_scs_d2

  USE yomhook, ONLY : lhook, dr_hook, jphook
!INTF_ON
  IMPLICIT NONE

  TYPE(rttov_options),                INTENT(IN)              :: opts
  TYPE(rttov_coef),                   INTENT(IN)              :: coef
  TYPE(rttov_chanprof),               INTENT(IN)              :: chanprof(:)
  LOGICAL(jplm),                      INTENT(IN)              :: thermal(:)
  LOGICAL(jplm),                      INTENT(IN)              :: solar(:)
  REAL(jprv),                         INTENT(IN)              :: brdf(:)
  REAL(jprv),                         INTENT(IN)              :: brdf_tl(:)
  REAL(jprv),                         INTENT(IN)              :: refl_norm(:)
  REAL(jprv),                         INTENT(IN)              :: diffuse_refl(:)
  REAL(jprv),                         INTENT(IN)              :: diffuse_refl_tl(:)
  REAL(jprv),                         INTENT(IN),   OPTIONAL  :: refl_cloud_top_in(:)
  REAL(jprv),                         INTENT(IN)              :: solar_spectrum(:)
  TYPE(rttov_profile_internal),       INTENT(IN)              :: profiles(:)
  TYPE(rttov_profile_internal),       INTENT(IN)              :: profiles_tl(:)
  TYPE(rttov_raytracing),             INTENT(IN)              :: raytracing
  TYPE(rttov_raytracing),             INTENT(IN)              :: raytracing_tl
  INTEGER(jpim),                      INTENT(IN)              :: maxncolumns
  TYPE(rttov_cloud_columns),          INTENT(IN)              :: cloud_columns
  TYPE(rttov_cloud_columns),          INTENT(IN)              :: cloud_columns_tl
  TYPE(rttov_transmission_aux),       INTENT(IN)              :: transmission_aux
  TYPE(rttov_transmission_aux),       INTENT(IN)              :: transmission_aux_tl
  TYPE(rttov_radiance_aux),           INTENT(IN)              :: auxrad_column
  TYPE(rttov_radiance_aux),           INTENT(INOUT)           :: auxrad_column_tl
  TYPE(rttov_radiance),               INTENT(IN)              :: rad
  TYPE(rttov_radiance),               INTENT(INOUT)           :: rad_tl
!INTF_END

  INTEGER(jpim) :: i, lev, col, lay, nchanprof, nlayers, nlevels
  LOGICAL(jplm) :: key_rad_only, do_scatt
  REAL(jprv)    :: refl, refl_norm_scat
  REAL(jprv)    :: rad_up_tl(profiles(1)%nlayers,0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: rad_down_tl(profiles(1)%nlayers,0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: rad_up_total_tl(0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: rad_down_total_tl(0:maxncolumns,SIZE(chanprof))
  REAL(jprv)    :: refl_cloud_top(SIZE(chanprof))
  REAL(jphook)  :: zhook_handle
!- End of header ------------------------------------------------------

  IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_SOLAR_TL',0_jpim,ZHOOK_HANDLE)

#define prof chanprof(i)%Prof
#define chan chanprof(i)%Chan

  !-------------------------------------------------------------------------------
  ! Initialise useful variables
  !-------------------------------------------------------------------------------

  nchanprof = SIZE(chanprof)
  nlayers = profiles(1)%nlayers
  nlevels = nlayers + 1

  do_scatt = opts%scatt%hydrometeors .OR. opts%scatt%aerosols
  key_rad_only = opts%pcrttov%enable_pcrttov .OR. do_scatt

  rad_up_tl(:,:,:)       = 0._jprv
  rad_down_tl(:,:,:)     = 0._jprv
  rad_up_total_tl(:,:)   = 0._jprv
  rad_down_total_tl(:,:) = 0._jprv

  IF (PRESENT(refl_cloud_top_in)) THEN
    refl_cloud_top(:) = refl_cloud_top_in(:)
  ELSE
    refl_cloud_top(:) = 0._jprv
  ENDIF

  !-------------------------------------------------------------------------------
  ! Calculate clear-sky Rayleigh scattering contribution from layers
  !-------------------------------------------------------------------------------
  IF (opts%rt_all%rayleigh_single_scatt) THEN
    CALL rayleigh_single_scattering_tl(raytracing, raytracing_tl, &
                                       profiles, profiles_tl, transmission_aux, &
                                       transmission_aux_tl)

    ! Add solar atmospheric contributions to the clear and cloudy columns
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE
      refl_norm_scat = COS(profiles(prof)%zenangle * deg2rad) * pi_r

      DO col = 0, cloud_columns%ncolumn(prof)
        auxrad_column_tl%rad_column(col,i) = auxrad_column_tl%rad_column(col,i) + &
                          rad_up_total_tl(col,i) + &
                          transmission_aux%solar_path1%tau_level(nlevels,col,i) * refl_norm_scat * &
                          (diffuse_refl(i) * &
                            (rad_down_total_tl(col,i) * &
                            transmission_aux%solar_path1%tau_level(nlevels,col,i) + &
                            2._jprv * auxrad_column%rad_down_solar(nlayers,col,i) * &
                            transmission_aux_tl%solar_path1%tau_level(nlevels,col,i)) + &
                          diffuse_refl_tl(i) * &
                          auxrad_column%rad_down_solar(nlayers,col,i) * &
                          transmission_aux%solar_path1%tau_level(nlevels,col,i))
      ENDDO
    ENDDO
  ENDIF

  !-------------------------------------------------------------------------------
  ! Solar surface contribution
  !-------------------------------------------------------------------------------
  IF (.NOT. do_scatt) &
    CALL solar_surface_contribution_tl(transmission_aux, transmission_aux_tl, &
                                       brdf, brdf_tl, auxrad_column_tl)

  !-------------------------------------------------------------------------------
  ! Calculate overcast radiances
  !-------------------------------------------------------------------------------
  IF (.NOT. key_rad_only) THEN
    col = 0_jpim
  !cdir nodep
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE
      IF (refl_cloud_top(i) > 0) THEN
        refl = refl_cloud_top(i)
      ELSE
        IF (coef%ff_cwn(chan) > overcast_albedo_wvn) THEN
          refl = overcast_albedo1 * pi_r
        ELSE
          refl = overcast_albedo2 * pi_r
        ENDIF
      ENDIF
      DO lay = 1, nlayers
        lev = lay + 1
        rad_tl%overcast(lay,i) = solar_spectrum(i) * refl / raytracing%pathsun(lay,prof) * &
                (transmission_aux_tl%solar_path2%tau_level(lev,col,i) - &
                transmission_aux%solar_path2%tau_level(lev,col,i) * &
                raytracing_tl%pathsun(lay,prof) / raytracing%pathsun(lay,prof)) + &
                rad_up_tl(lay,col,i) + &
                refl / raytracing%pathsat(lay,prof) * &
                transmission_aux%solar_path1%tau_level(lev,col,i) * &
                (rad_down_tl(lay,col,i) * &
                transmission_aux%solar_path1%tau_level(lev,col,i) - &
                auxrad_column%rad_down_solar(lay,col,i) * &
                raytracing_tl%pathsat(lay,prof) / raytracing%pathsat(lay,prof) * &
                transmission_aux%solar_path1%tau_level(lev,col,i) + &
                auxrad_column%rad_down_solar(lay,col,i) * &
                2_jpim * transmission_aux_tl%solar_path1%tau_level(lev,col,i))
      ENDDO
    ENDDO
  ENDIF

  !-------------------------------------------------------------------------------
  ! Calculate total radiance
  !-------------------------------------------------------------------------------
  WHERE (solar(1:nchanprof) .AND. .NOT. thermal(1:nchanprof)) &
    rad_tl%clear(1:nchanprof) = auxrad_column_tl%rad_column(0,1:nchanprof)

  IF (do_scatt) THEN
    !---------------------------------------------------
    ! Calculate Rayleigh single scattering radiance contributions for scattering radiances
    !---------------------------------------------------
    IF (opts%rt_all%rayleigh_single_scatt) THEN
      DO i = 1, nchanprof
        IF (thermal(i) .OR. .NOT. solar(i)) CYCLE

        DO col = 1, cloud_columns%ncolumn(prof)
          rad_tl%total(i) = rad_tl%total(i) + &
                  auxrad_column_tl%rad_column(col,i) * &
                  (cloud_columns%xcol(col+1,prof) - cloud_columns%xcol(col,prof)) + &
                  auxrad_column%rad_column(col,i) * &
                  (cloud_columns_tl%xcol(col+1,prof) - cloud_columns_tl%xcol(col,prof))
        ENDDO

        rad_tl%total(i) = rad_tl%total(i) + &
                          auxrad_column_tl%rad_column(0,i) * cloud_columns%xcolclr(prof) + &
                          auxrad_column%rad_column(0,i) * cloud_columns_tl%xcolclr(prof)
      ENDDO
    ENDIF
  ELSE
    !---------------------------------------------------
    ! Calculate total radiance (clear case/simple cloud)
    !---------------------------------------------------
    DO i = 1, nchanprof
      IF (thermal(i) .OR. .NOT. solar(i)) CYCLE

      lay = profiles(prof)%ctp_layer
      rad_tl%cloudy(i) = rad_tl%overcast(lay-1,i) + &
                          profiles_tl(prof)%ctp_pfraction * &
                          (rad%overcast(lay,i) - rad%overcast(lay-1,i)) + &
                          profiles(prof)%ctp_pfraction * &
                          (rad_tl%overcast(lay,i) - rad_tl%overcast(lay-1,i))

      rad_tl%total(i) = rad_tl%clear(i) + &
                        profiles_tl(prof)%cfraction * &
                        (rad%cloudy(i) - rad%clear(i)) + &
                        profiles(prof)%cfraction * &
                        (rad_tl%cloudy(i) - rad_tl%clear(i))
    ENDDO
  ENDIF

  IF (LHOOK) CALL DR_HOOK('RTTOV_INTEGRATE_SOLAR_TL',1_jpim,ZHOOK_HANDLE)

CONTAINS

  SUBROUTINE rayleigh_single_scattering_tl(raytracing, raytracing_tl, &
                                           profiles, profiles_tl, transmission_aux, &
                                           transmission_aux_tl)

    TYPE(rttov_raytracing),       INTENT(IN)    :: raytracing, raytracing_tl
    TYPE(rttov_profile_internal), INTENT(IN)    :: profiles(:), profiles_tl(:)
    TYPE(rttov_transmission_aux), INTENT(IN)    :: transmission_aux, transmission_aux_tl

    INTEGER(jpim) :: i, col, lay, lev
    REAL(jprv) :: wlm, ss_param
    REAL(jprv) :: cossat, cossol, cosazi, cosscata_term1, cosscata_term2, cosscata
    REAL(jprv) :: gamma, ray_phase, solar_src, solar_src_updn
    REAL(jprv) :: cosscata_term1_tl, cosscata_term2_tl, cosscata_tl
    REAL(jprv) :: ray_phase_tl, solar_src_tl, solar_src_updn_tl
    REAL(jprv) :: rayrad_up_tl(0:nlayers,0:maxncolumns), rayrad_dn_tl(0:nlayers,0:maxncolumns)

    DO i = 1, nchanprof
      wlm = 10000._jprv / coef%ff_cwn(chan)    ! Wavelength in microns
      IF (.NOT. solar(i) .OR. wlm > opts%rt_all%rayleigh_max_wavelength) CYCLE

      ! Calculate layer-independent scattering parameter

      IF (coef%fmv_model_ver <= 9) THEN
        IF (wlm < ray_scs_wlm) THEN
          ss_param = ray_scs_a1x * wlm ** (ray_scs_b1 + ray_scs_c1 * wlm + ray_scs_d1/wlm)
        ELSE
          ss_param = ray_scs_a2x * wlm ** (ray_scs_b2 + ray_scs_c2 * wlm + ray_scs_d2/wlm)
        ENDIF
        ss_param = ss_param * 0.01_jprv ** 2_jpim * 1.E3_jprv * 100._jprv * na * z4pi_r / (gravity * mair)
      ELSE
        ss_param = coef%ss_rayleigh_ext(chan) * (ray_ts / ray_ps) * rgc * z4pi_r / (gravity * mair)
      ENDIF
      gamma = coef%rayleigh_depol_gamma(chan)

      rayrad_up_tl(0,:) = 0._jprv
      rayrad_dn_tl(0,:) = 0._jprv

      ! Sum contributions from atmospheric layers
      DO lev = 2, nlevels
        lay = lev - 1

        ! Skip calculation for whole layers above min pressure
        IF (profiles(prof)%p_half(lev) <= opts%rt_all%rayleigh_min_pressure) THEN
          rayrad_up_tl(lay,:) = 0._jprv
          rayrad_dn_tl(lay,:) = 0._jprv
          CYCLE
        ENDIF

        solar_src = solar_spectrum(i) * &
              (profiles(prof)%p_half(lev) - profiles(prof)%p_half(lev-1)) * &
              raytracing%pathsat(lay, prof) * ss_param

        ! cosine of scattering angle - raytracing%zasat/zasun contain the sine of the angles
        ! NB there are no perturbations around the profile angles, but the raytracing angles
        !    are dependent on atmospheric parameters.
        cossat = 1._jprv - raytracing%zasat(lay, prof) * raytracing%zasat(lay, prof)
        cossol = 1._jprv - raytracing%zasun(lay, prof) * raytracing%zasun(lay, prof)
        cosscata_term1 = SQRT(cossat * cossol)
        cosazi = COS((profiles(prof)%azangle - profiles(prof)%sunazangle)*deg2rad)
        cosscata_term2 = raytracing%zasat(lay, prof) * raytracing%zasun(lay, prof) * cosazi
        cosscata = - cosscata_term1 - cosscata_term2
        ray_phase = (1._jprv + 3._jprv * gamma + (1._jprv - gamma) * cosscata**2) * &
                    0.75_jprv / (1._jprv + 2._jprv * gamma)
        solar_src_updn = solar_src * ray_phase

        solar_src_tl = solar_spectrum(i) * &
            (profiles(prof)%p_half(lev) - profiles(prof)%p_half(lev-1)) * &
            raytracing_tl%pathsat(lay, prof) * ss_param

        IF (opts%interpolation%pressure_gradients) THEN
          solar_src_tl = solar_src_tl + &
              solar_spectrum(i) * &
              (profiles_tl(prof)%p_half(lev) - profiles_tl(prof)%p_half(lev-1)) * &
              raytracing%pathsat(lay, prof) * ss_param
        ENDIF

        cosscata_term1_tl = -1._jprv / cosscata_term1 * &
                            (raytracing_tl%zasat(lay, prof) * raytracing%zasat(lay, prof) * cossol + &
                             raytracing_tl%zasun(lay, prof) * raytracing%zasun(lay, prof) * cossat)
        cosscata_term2_tl = raytracing_tl%zasat(lay, prof) * raytracing%zasun(lay, prof) * cosazi + &
                            raytracing_tl%zasun(lay, prof) * raytracing%zasat(lay, prof) * cosazi

        cosscata_tl = - cosscata_term1_tl - cosscata_term2_tl
        ray_phase_tl = 2._jprv * (1._jprv - gamma) * cosscata_tl * cosscata * &
                       0.75_jprv / (1._jprv + 2._jprv * gamma)
        solar_src_updn_tl = solar_src_tl * ray_phase + solar_src * ray_phase_tl

        DO col = 0, cloud_columns%ncolumn(prof)
          rayrad_up_tl(lay,col) = rayrad_up_tl(lay-1,col) + &
                                  transmission_aux_tl%solar_path2%tau_level(lev-1,col,i) * solar_src_updn + &
                                  transmission_aux%solar_path2%tau_level(lev-1,col,i) * solar_src_updn_tl

          IF (transmission_aux%solar_path1%tau_level(lev-1,col,i) > min_tau) THEN
            rayrad_dn_tl(lay,col) = rayrad_dn_tl(lay-1,col) + &
                                    (solar_src_updn_tl * transmission_aux%solar_path2%tau_level(lev-1,col,i) + &
                                    solar_src_updn * transmission_aux_tl%solar_path2%tau_level(lev-1,col,i) - &
                                    solar_src_updn * transmission_aux%solar_path2%tau_level(lev-1,col,i) * &
                                    3_jpim * transmission_aux_tl%solar_path1%tau_level(lev-1,col,i) / &
                                    transmission_aux%solar_path1%tau_level(lev-1,col,i)) / &
                                    transmission_aux%solar_path1%tau_level(lev-1,col,i) ** 3_jpim
          ELSE
            rayrad_dn_tl(lay,col) = rayrad_dn_tl(lay-1,col)
          ENDIF
        ENDDO
      ENDDO

      ! Add Rayleigh contributions to radiance totals
      DO col = 0, cloud_columns%ncolumn(prof)
        rad_up_tl(:,col,i)       = rayrad_up_tl(1:nlayers,col)
        rad_up_total_tl(col,i)   = rayrad_up_tl(nlayers,col)
        rad_down_tl(:,col,i)     = rayrad_dn_tl(1:nlayers,col)
        rad_down_total_tl(col,i) = rayrad_dn_tl(nlayers,col)
      ENDDO

    ENDDO
  END SUBROUTINE rayleigh_single_scattering_tl

  SUBROUTINE solar_surface_contribution_tl(transmission_aux, transmission_aux_tl, &
                                           brdf, brdf_tl, auxrad_column_tl)

    TYPE(rttov_transmission_aux), INTENT(IN)    :: transmission_aux, transmission_aux_tl
    REAL(jprv),                   INTENT(IN)    :: brdf(:), brdf_tl(:)
    TYPE(rttov_radiance_aux),     INTENT(INOUT) :: auxrad_column_tl

    INTEGER(jpim) :: i, col

    col = 0
    DO i = 1, nchanprof
      IF (.NOT. solar(i)) CYCLE
      auxrad_column_tl%rad_column(col,i) = auxrad_column_tl%rad_column(col,i) + &
            solar_spectrum(i) * refl_norm(i) * &
            (brdf_tl(i) * transmission_aux%solar_path2%tau_level(nlevels,col,i) + &
            brdf(i) * transmission_aux_tl%solar_path2%tau_level(nlevels,col,i))
    ENDDO
  END SUBROUTINE solar_surface_contribution_tl

END SUBROUTINE rttov_integrate_solar_tl
